<?php

namespace XF2SEO\SEOPackage\Admin\Controller;

use XF\Mvc\FormAction;
use XF\Mvc\ParameterBag;
use XF\ControllerPlugin\Delete;
use XF\Mvc\Reply\AbstractReply;
use XF\Admin\Controller\AbstractController;

class Keyword extends AbstractController
{
    public function actionIndex()
    {
        $finder = $this->finder('XF2SEO\SEOPackage:Keyword');
        $finder->order('execute_order');

        $page = $this->filterPage();
        $perPage = 30;

        $total = $finder->total();
        $this->assertValidPage($page, $perPage, $total, $this->getLinkPrefix());

        $keywords = $finder->limitByPage($page, $perPage)->fetch();

        return $this->view(
            '',
            'seo_package_keyword_list',
            [
                'keywords' => $keywords,
                'linkPrefix' => $this->getLinkPrefix(),
                'page' => $page,
                'perPage' => $perPage,
                'total' => $total,
            ]
        );
    }

    public function actionAdd()
    {
        return $this->getKeywordForm($this->getNewKeyword());
    }

    public function actionEdit(ParameterBag $params)
    {
        $keyword = $this->assertKeywordExists($params['keyword_id']);

        return $this->getKeywordForm($keyword);
    }

    public function actionSave(ParameterBag $params)
    {
        if ($params['keyword_id'] > 0) {
            $keyword = $this->assertKeywordExists($params['keyword_id']);
        } else {
            $keyword = $this->getNewKeyword();
        }

        $this->saveKeywordProcess($keyword)->run();

        return $this->redirect(
            $this->buildLink($this->getLinkPrefix())
            . $this->buildLinkHash($keyword->keyword_id)
        );
    }

    public function actionDelete(ParameterBag $params)
    {
        $keyword = $this->assertKeywordExists($params['keyword_id']);

        /** @var Delete $deletePlugin */
        $deletePlugin = $this->plugin('XF:Delete');

        return $deletePlugin->actionDelete(
            $keyword,
            $this->buildLink($this->getLinkPrefix() . '/delete', $keyword),
            $this->buildLink($this->getLinkPrefix() . '/edit', $keyword),
            $this->buildLink($this->getLinkPrefix()),
            $keyword->keyword
        );
    }

    protected function getLinkPrefix(): string
    {
        return 'tsp-keywords';
    }

    protected function saveKeywordProcess(\XF2SEO\SEOPackage\Entity\Keyword $keyword): FormAction
    {
        $formAction = $this->formAction();

        $input = $this->filter([
            'keyword' => 'str',
            'replacement' => 'str',
            'description' => 'str',
            'target_url' => 'str',
            'active' => 'bool',
            'options' => 'array',
            'execute_order' => 'uint',
        ]);

        $formAction->basicEntitySave($keyword, $input);

        return $formAction;
    }

    protected function getNewKeyword(): \XF2SEO\SEOPackage\Entity\Keyword
    {
        /** @var \XF2SEO\SEOPackage\Entity\Keyword $keyword */
        $keyword = $this->em()->create('XF2SEO\SEOPackage:Keyword');

        return $keyword;
    }

    protected function getKeywordForm(\XF2SEO\SEOPackage\Entity\Keyword $keyword): AbstractReply
    {
        return $this->view(
            '',
            'seo_package_keyword_edit',
            [
                'keyword' => $keyword,
                'linkPrefix' => $this->getLinkPrefix(),
            ]
        );
    }

    /**
     * @param mixed $id
     * @return \XF2SEO\SEOPackage\Entity\Keyword
     * @throws \XF\Mvc\Reply\Exception
     */
    protected function assertKeywordExists($id): \XF2SEO\SEOPackage\Entity\Keyword
    {
        /** @var \XF2SEO\SEOPackage\Entity\Keyword $keyword */
        $keyword = $this->assertRecordExists(
            'XF2SEO\SEOPackage:Keyword',
            $id
        );

        return $keyword;
    }
}
